//
// Ephi - simulation of magnetic fields and particles
// Copyright (C) 2007 Indrek Mandre <indrek(at)mare.ee>
// For more information please see http://www.mare.ee/indrek/ephi/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "statics.hpp"
#include "screen.hpp"
#include "dynamics.hpp"
#include "consts.hpp"
#include "polywell_conf.hpp"

// test for b field in the center of a quite fat solenoid
// trying to simulate that in ephi through multiple coils
// in a rectangular configuration

#define RADIUS 0.2
#define WIDTH 0.1
#define I 1.0
#define OFFSET 0

// from http://www.netdenizen.com/emagnet/solenoids/solenoidonaxis.htm
#define KE (MAGNETIC_CONSTANT*SPEED_OF_LIGHT*SPEED_OF_LIGHT)
#define M0 (4.0*PREC_PI*MAGNETIC_CONSTANT)

static prec_t Bz (prec_t d)
{
  prec_t l = WIDTH;
  prec_t x1 = d - l*0.5;
  prec_t x2 = d + l*0.5;
  prec_t r1 = RADIUS - WIDTH/2;
  prec_t r2 = RADIUS + WIDTH/2;
  return ((M0*I)/(2*WIDTH*WIDTH)) *
      (x2 * prec_t_log((prec_t_sqrt(r2*r2+x2*x2)+r2)/(prec_t_sqrt(r1*r1+x2*x2)+r1)) -
          x1 * prec_t_log((prec_t_sqrt(r2*r2+x1*x1)+r2)/(prec_t_sqrt(r1*r1+x1*x1)+r1)));
}

int main (int argc, char *argv[])
{
  for ( int k = 1; k <= 32; k++ )
    {
      Statics statics;
      ElectroDynamics dyn(statics);
      ShapedCoilFactory(k).make (statics, vect3d(0, 0, 0), vect3d(0, 0, 1), RADIUS, WIDTH/2, I, 0);
      vect3d bf;
      dyn.getBField (vect3d(0, 0, OFFSET), bf);
      prec_t tb = Bz(OFFSET);
      printf ("%d: %.6f%%\n", k, prec2double(100*fabs(tb-bf.z)/tb));

#if 0
      FILE *fout = fopen ("polywell.inc", "w");
      fprintf (fout, "%s", statics.get_povray_script().c_str());
      fclose (fout);
#endif
    }

  return 0;
}

