//
// Ephi - simulation of magnetic fields and particles
// Copyright (C) 2007 Indrek Mandre <indrek(at)mare.ee>
// For more information please see http://www.mare.ee/indrek/ephi/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "scene.hpp"
#include "statics.hpp"
#include "screen.hpp"
#include "dynamics.hpp"
#include "polywell_conf.hpp"

// visualize electric potential and electric field of a line of charge with linear charge distribution

int main (int argc, char *argv[])
{
  Statics statics;
  ElectroDynamics dyn(statics);

  statics.addStaticElement (new StaticLineCharge (vect3d(-0.25, 0, 0), vect3d(0.25, 0, 0), 0, 1e-9, 0.025));

  fprintf (stderr, "Calculating...\n");
  Screen screen(800, 800, 0.9, 0.9);
  Scene scene(dyn);
  scene.set_coloring (20, false);
  scene.calc (screen, Scene::CALC_POTENTIAL);
  scene.render_map (screen);
  screen.write ("test14_pot.bmp");

  scene.calc (screen, Scene::CALC_EFIELD);
  scene.set_coloring (0, true);
  scene.render_map (screen);
  screen.write ("test14_efield_log.bmp");

  scene.set_coloring (20, true);
  scene.render_map (screen);
  screen.write ("test14_efield_log_stepped.bmp");

  scene.set_coloring (0, true);
  scene.render_lic (screen);
  screen.write ("test14_lic.bmp");

  scene.set_coloring (0, false);
  scene.render_map (screen);
  screen.write ("test14_efield.bmp");

  return 0;
}

