//
// Ephi - simulation of magnetic fields and particles
// Copyright (C) 2007 Indrek Mandre <indrek(at)mare.ee>
// For more information please see http://www.mare.ee/indrek/ephi/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "statics.hpp"
#include "screen.hpp"
#include "dynamics.hpp"

// an electron trapped in a magnetic field from a circular current element

#define TIME_TO_RUN (2e-6/5)
#define CURRENT 100000.0
#define RADIUS 0.2
#define SPEED 350

int main (int argc, char *argv[])
{
  Statics statics;
  ElectroDynamics dyn(statics);
  statics.addStaticElement (new StaticRing(CURRENT, vect3d(0, 0, 0), RADIUS, 0, vect3d(0.0, 0.0, 1.0)));

  dyn.inject_electron (vect3d(-1.5, 0, 0), vect3d(1, 0, 0), SPEED);
  dyn.setDTM (0.01); // don't want to wait too long

  Screen screen(800, 800, 4.0, 4.0);
  screen.draw_axes();

  prec_t startmag = dyn.get_v(0).length();

  statics.draw (screen, Screen::BLACK);

#define PIXEL_INTERVAL 0.0000000005
  prec_t cp = 0;

  while ( true )
    {
      prec_t et = dyn.get_elapsed_time ();
      if ( et >= cp )
        {
          screen.set_pixel (dyn.get_pos(0), Screen::RED);
          cp += PIXEL_INTERVAL;
          fprintf (stderr, "\rCalculating: %.2f%%", prec2double(100.0 * dyn.get_elapsed_time() / TIME_TO_RUN));
          fflush (stderr);
//          dyn.print();
        }

      if ( et >= TIME_TO_RUN )
          break;
      dyn.step ();
    }
  fprintf (stderr, "\r");
  fflush (stderr);
  
  screen.write ("test5.bmp");

  prec_t diff = fabs (startmag - dyn.get_v(0).length());
  if ( diff > startmag / 1000 )
      printf ("ERROR=speed changed significantly, what gives?\n");
  else
      printf ("OK=generated test5.bmp\n");
  printf ("MARGIN=%g\n", prec2double(diff));

  return 0;
}

